@@ -451,8 +451,8 @@ void at_initiator_annotated::send_end_rsp_method(void) // send end response meth
 //=============================================================================
 
 void at_initiator_annotated::invalidate_direct_mem_ptr  // invalidate_direct_mem_ptr
-( sc_dt::uint64 start_range                             // start range
-, sc_dt::uint64 end_range                               // end range
+( sc_dt::uint64 /*start_range*/                         // start range
+, sc_dt::uint64 /*end_range*/                           // end range
 ) 
 {
   std::ostringstream       msg;                         // log message
