@@ -496,8 +496,8 @@ void select_initiator::send_end_rsp_method(void)    // send end response method
 //=============================================================================
 
 void select_initiator::invalidate_direct_mem_ptr  // invalidate_direct_mem_ptr
-( sc_dt::uint64 start_range                       // start range
-, sc_dt::uint64 end_range                         // end range
+( sc_dt::uint64 /*start_range*/                   // start range
+, sc_dt::uint64 /*end_range*/                     // end range
 )
 {
   std::ostringstream       msg;                   // log message
