@@ -33,7 +33,7 @@
 
       Name, Affiliation, Date:
   Description of Modification:
-    
+
  *****************************************************************************/
 
 
@@ -73,21 +73,26 @@
 #include "sysc/datatypes/fx/sc_fix.h"
 #include "sysc/datatypes/fx/scfx_other_defs.h"
 
+#include <sstream>
 
-namespace sc_dt
-{
+// explicit template instantiations
+namespace sc_core {
+template class SC_API sc_vpool<sc_dt::sc_int_bitref>;
+template class SC_API sc_vpool<sc_dt::sc_int_subref>;
+} // namespace sc_core
+
+namespace sc_dt {
 
 // to avoid code bloat in sc_int_concref<T1,T2>
 
 void
 sc_int_concref_invalid_length( int length )
 {
-    char msg[BUFSIZ];
-    std::sprintf( msg,
-	     "sc_int_concref<T1,T2> initialization: length = %d "
-	     "violates 1 <= length <= %d",
-	     length, SC_INTWIDTH );
-    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg );
+    std::stringstream msg;
+    msg << "sc_int_concref<T1,T2> initialization: length = " << length
+        << "violates 1 <= length <= " << SC_INTWIDTH;
+    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
+    sc_core::sc_abort(); // can't recover from here
 }
 
 
@@ -151,7 +156,7 @@ sc_int_bitref::scan( ::std::istream& is )
 // ----------------------------------------------------------------------------
 
 bool sc_int_subref_r::concat_get_ctrl( sc_digit* dst_p, int low_i ) const
-{    
+{
     int       dst_i;       // Word in dst_p now processing.
     int       end_i;       // Highest order word in dst_p to process.
     int       high_i;      // Index of high order bit in dst_p to set.
@@ -197,7 +202,7 @@ bool sc_int_subref_r::concat_get_ctrl( sc_digit* dst_p, int low_i ) const
 
 
 bool sc_int_subref_r::concat_get_data( sc_digit* dst_p, int low_i ) const
-{    
+{
     int       dst_i;      // Word in dst_p now processing.
     int       end_i;      // Highest order word in dst_p to process.
     int       high_i;     // Index of high order bit in dst_p to set.
@@ -217,8 +222,8 @@ bool sc_int_subref_r::concat_get_data( sc_digit* dst_p, int low_i ) const
 
     // PROCESS THE FIRST WORD:
 
-    mask = ~(-1 << left_shift);
-    dst_p[dst_i] = (sc_digit)((dst_p[dst_i] & mask) | 
+    mask = ~(~UINT_ZERO << left_shift);
+    dst_p[dst_i] = (sc_digit)((dst_p[dst_i] & mask) |
 		((val << left_shift) & DIGIT_MASK));
 
     switch ( end_i - dst_i )
@@ -265,8 +270,8 @@ bool sc_int_subref_r::concat_get_data( sc_digit* dst_p, int low_i ) const
 sc_core::sc_vpool<sc_int_subref> sc_int_subref::m_pool(9);
 
 // assignment operators
-  
-sc_int_subref& 
+
+sc_int_subref&
 sc_int_subref::operator = ( int_type v )
 {
     int_type val = m_obj_p->m_val;
@@ -363,34 +368,32 @@ sc_int_subref::scan( ::std::istream& is )
 void
 sc_int_base::invalid_length() const
 {
-    char msg[BUFSIZ];
-    std::sprintf( msg,
-	     "sc_int[_base] initialization: length = %d violates "
-	     "1 <= length <= %d",
-	     m_len, SC_INTWIDTH );
-    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg );
+    std::stringstream msg;
+    msg << "sc_int[_base] initialization: length = " << m_len
+        << " violates 1 <= length <= " << SC_INTWIDTH;
+    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
+    sc_core::sc_abort(); // can't recover from here
 }
 
 void
 sc_int_base::invalid_index( int i ) const
 {
-    char msg[BUFSIZ];
-    std::sprintf( msg,
-	     "sc_int[_base] bit selection: index = %d violates "
-	     "0 <= index <= %d",
-	     i, m_len - 1 );
-    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg );
+    std::stringstream msg;
+    msg << "sc_int[_base] bit selection: index = " << i
+        << " violates 0 <= index <= " << (m_len - 1);
+    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
+    sc_core::sc_abort(); // can't recover from here
 }
 
 void
 sc_int_base::invalid_range( int l, int r ) const
 {
-    char msg[BUFSIZ];
-    std::sprintf( msg,
-	     "sc_int[_base] part selection: left = %d, right = %d violates "
-	     "%d >= left >= right >= 0",
-	     l, r, m_len - 1 );
-    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg );
+    std::stringstream msg;
+    msg << "sc_int[_base] part selection: "
+        << "left = " << l << ", right = " << r << " violates "
+        << (m_len-1) << " >= left >= right >= 0";
+    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
+    sc_core::sc_abort(); // can't recover from here
 }
 
 
@@ -399,16 +402,15 @@ sc_int_base::check_value() const
 {
     int_type limit = (int_type) 1 << ( m_len - 1 );
     if( m_val < -limit || m_val >= limit ) {
-	char msg[BUFSIZ];
-	std::sprintf( msg, "sc_int[_base]: value does not fit into a length of %d",
-		 m_len );
-	SC_REPORT_WARNING( sc_core::SC_ID_OUT_OF_BOUNDS_, msg );
+        std::stringstream msg;
+        msg << "sc_int[_base]: value does not fit into a length of " << m_len;
+        SC_REPORT_WARNING( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
     }
 }
 
 
 // constructors
-sc_int_base::sc_int_base( const sc_bv_base& v ) 
+sc_int_base::sc_int_base( const sc_bv_base& v )
     : m_val(0), m_len( v.length() ), m_ulen( SC_INTWIDTH - m_len )
 {
     check_length();
@@ -470,7 +472,7 @@ sc_int_base::sc_int_base( const sc_unsigned& a )
 
 // assignment operators
 
-sc_int_base& 
+sc_int_base&
 sc_int_base::operator = ( const sc_signed& a )
 {
     int minlen = sc_min( m_len, a.length() );
@@ -487,7 +489,7 @@ sc_int_base::operator = ( const sc_signed& a )
     return *this;
 }
 
-sc_int_base& 
+sc_int_base&
 sc_int_base::operator = ( const sc_unsigned& a )
 {
     int minlen = sc_min( m_len, a.length() );
@@ -543,21 +545,20 @@ sc_int_base::operator = ( const char* a )
 	SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
 			 "character string is zero" );
     }
-    if( *a == 0 ) {
+    else if( *a == 0 ) {
 	SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
 			 "character string is empty" );
     }
-    try {
+    else try {
 	int len = m_len;
 	sc_fix aa( a, len, len, SC_TRN, SC_WRAP, 0, SC_ON );
 	return this->operator = ( aa );
-    } catch( sc_core::sc_report ) {
-	char msg[BUFSIZ];
-	std::sprintf( msg, "character string '%s' is not valid", a );
-	SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
-	// never reached
-	return *this;
+    } catch( const sc_core::sc_report & ) {
+        std::stringstream msg;
+        msg << "character string '" << a << "' is not valid";
+        SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
     }
+    return *this;
 }
 
 // explicit conversion to character string
@@ -609,7 +610,7 @@ sc_int_base::xor_reduce() const
 
 
 bool sc_int_base::concat_get_ctrl( sc_digit* dst_p, int low_i ) const
-{    
+{
     int        dst_i;       // Word in dst_p now processing.
     int        end_i;       // Highest order word in dst_p to process.
     int        left_shift;  // Left shift for val.
@@ -619,7 +620,7 @@ bool sc_int_base::concat_get_ctrl( sc_digit* dst_p, int low_i ) const
     left_shift = low_i % BITS_PER_DIGIT;
     end_i = (low_i + (m_len-1)) / BITS_PER_DIGIT;
 
-    mask = ~(-1 << left_shift);
+    mask = ~(~UINT_ZERO << left_shift);
     dst_p[dst_i] = (sc_digit)(dst_p[dst_i] & mask);
 	dst_i++;
 	for ( ; dst_i <= end_i; dst_i++ ) dst_p[dst_i] = 0;
@@ -642,7 +643,7 @@ bool sc_int_base::concat_get_ctrl( sc_digit* dst_p, int low_i ) const
 //   low_i =  first bit within dst_p to be set.
 //------------------------------------------------------------------------------
 bool sc_int_base::concat_get_data( sc_digit* dst_p, int low_i ) const
-{    
+{
     int        dst_i;       // Word in dst_p now processing.
     int        end_i;       // Highest order word in dst_p to process.
     int        high_i;      // Index of high order bit in dst_p to set.
@@ -662,14 +663,14 @@ bool sc_int_base::concat_get_data( sc_digit* dst_p, int low_i ) const
 
     if ( m_len < 64 )
     {
-	mask = ~((uint_type)-1 << m_len);
+	mask = ~(~UINT_ZERO << m_len);
         val &=  mask;
     }
 
     // PROCESS THE FIRST WORD:
 
-    mask = (-1 << left_shift);
-    dst_p[dst_i] = (sc_digit)((dst_p[dst_i] & ~mask) | 
+    mask = (~UINT_ZERO << left_shift);
+    dst_p[dst_i] = (sc_digit)((dst_p[dst_i] & ~mask) |
 		((val <<left_shift) & DIGIT_MASK));
     switch ( end_i - dst_i )
     {
