@@ -594,8 +594,8 @@ and_on_help(small_type us,
             const sc_digit *vd)
 {
 
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -623,8 +623,8 @@ and_on_help(small_type us,
     }
     else {  // case 3
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -648,7 +648,7 @@ and_on_help(small_type us,
 
     if (us > 0) { // case 4
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -665,7 +665,7 @@ and_on_help(small_type us,
     }
     else {  // case 5
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -695,8 +695,8 @@ or_on_help(small_type us,
            const sc_digit *vd)
 {
   
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -722,8 +722,8 @@ or_on_help(small_type us,
     }
     else {  // case 4
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -747,7 +747,7 @@ or_on_help(small_type us,
 
     if (us > 0) { // case 5
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -766,7 +766,7 @@ or_on_help(small_type us,
     }
     else {  // case 6
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -798,8 +798,8 @@ xor_on_help(small_type us,
             const sc_digit *vd)
 {
   
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -828,8 +828,8 @@ xor_on_help(small_type us,
     }
     else {  // case 4
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -852,7 +852,7 @@ xor_on_help(small_type us,
 
     if (us > 0) { // case 5
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -871,7 +871,7 @@ xor_on_help(small_type us,
     }
     else {  // case 6
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
