@@ -26,41 +26,38 @@
  CHANGE LOG APPEARS AT THE END OF THE FILE
  *****************************************************************************/
 
+#define SC_DISABLE_API_VERSION_CHECK // for in-library sc_ver.h inclusion
 
 #include "sysc/kernel/sc_cmnhdr.h"
 #include "sysc/kernel/sc_externs.h"
 #include "sysc/kernel/sc_except.h"
-#include "sysc/utils/sc_iostream.h"
+#include "sysc/kernel/sc_ver.h"
 #include "sysc/utils/sc_report.h"
 #include "sysc/utils/sc_report_handler.h"
 #include "sysc/utils/sc_utils_ids.h"
+
+#include <algorithm>
+#include <cstring>
+#include <sstream>
 #include <vector>
 
 namespace sc_core {
 
 extern void pln();
 
-static int    argc_copy;	// Copy of argc value passed to sc_elab_and_sim.
-static char** argv_copy;	// Copy of argv value passed to sc_elab_and_sim.
-
-static
-inline
-void
-message_function( const char* s )
-{
-    ::std::cout << "\n" << s << ::std::endl;
-}
+static int    argc_orig;  // Original argc value passed to sc_elab_and_sim.
+static char** argv_orig;  // Original argv value passed to sc_elab_and_sim.
 
 bool sc_in_action = false;
 
-int sc_argc() 
+int sc_argc()
 {
-    return argc_copy;
+    return argc_orig;
 }
 
-const char* const* sc_argv() 
+const char* const* sc_argv()
 {
-    return argv_copy;
+    return argv_orig;
 }
 
 
@@ -68,11 +65,17 @@ int
 sc_elab_and_sim( int argc, char* argv[] )
 {
     int status = 1;
-    argc_copy = argc;
-    argv_copy = argv;
-    std::vector<char*> argv_call;
-    for ( int i = 0; i < argc; i++ ) 
-        argv_call.push_back(argv[i]);
+    argc_orig = argc;
+    argv_orig = argv;
+
+    // Copy argv into a new structure to prevent sc_main from modifying the
+    // result returned from sc_argv.
+    std::vector<char*> argv_copy(argc + 1, static_cast<char*>(NULL));
+    for ( int i = 0; i < argc; ++i ) {
+        std::size_t size = std::strlen(argv[i]) + 1;
+        argv_copy[i] = new char[size];
+        std::copy(argv[i], argv[i] + size, argv_copy[i]);
+    }
 
     try
     {
@@ -81,6 +84,8 @@ sc_elab_and_sim( int argc, char* argv[] )
         // Perform initialization here
         sc_in_action = true;
 
+        // copy array of pointers to keep allocated pointers for later release
+        std::vector<char*> argv_call = argv_copy;
         status = sc_main( argc, &argv_call[0] );
 
         // Perform cleanup here
@@ -88,34 +93,40 @@ sc_elab_and_sim( int argc, char* argv[] )
     }
     catch( const sc_report& x )
     {
-	message_function( x.what() );
+        sc_report_handler::get_handler()
+            ( x, sc_report_handler::get_catch_actions() );
     }
     catch( ... )
     {
         // translate other escaping exceptions
         sc_report*  err_p = sc_handle_exception();
-        if( err_p ) message_function( err_p->what() );
+        if( err_p )
+            sc_report_handler::get_handler()
+                ( *err_p, sc_report_handler::get_catch_actions() );
         delete err_p;
     }
 
-    // IF DEPRECATION WARNINGS WERE ISSUED TELL THE USER HOW TO TURN THEM OFF 
+    for ( int i = 0; i < argc; ++i ) {
+        delete[] argv_copy[i];
+    }
+
+    // IF DEPRECATION WARNINGS WERE ISSUED TELL THE USER HOW TO TURN THEM OFF
 
     if ( sc_report_handler::get_count( SC_ID_IEEE_1666_DEPRECATION_ ) > 0 )
     {
         std::stringstream ss;
 
-#       define MSGNL  "\n             "
-#       define CODENL "\n  "
+        const char MSGNL[] = "\n             ";
+        const char CODENL[] = "\n  ";
 
-        ss <<
-          "You can turn off warnings about" MSGNL
-          "IEEE 1666 deprecated features by placing this method call" MSGNL
-          "as the first statement in your sc_main() function:\n" CODENL
-          "sc_core::sc_report_handler::set_actions( "
-          "\"" << SC_ID_IEEE_1666_DEPRECATION_ << "\"," CODENL
-          "                                         " /* indent param */
-          "sc_core::SC_DO_NOTHING );"
-          << std::endl;
+        ss << "You can turn off warnings about" << MSGNL
+           << "IEEE 1666 deprecated features by placing this method call" << MSGNL
+           << "as the first statement in your sc_main() function:\n" << CODENL
+           << "sc_core::sc_report_handler::set_actions( "
+           << "\"" << SC_ID_IEEE_1666_DEPRECATION_ << "\"," << CODENL
+           << "                                         " /* indent param */
+           << "sc_core::SC_DO_NOTHING );"
+           << std::endl;
 
         SC_REPORT_INFO( SC_ID_IEEE_1666_DEPRECATION_, ss.str().c_str() );
     }
