@@ -53,6 +53,7 @@
 
 #include <cstdlib>
 #include <cstring>
+#include <limits>
 #include <vector>
 
 #include "sysc/kernel/sc_simcontext.h"
@@ -506,7 +507,7 @@ vcd_sc_unsigned_trace::write(FILE* f)
     static std::vector<char> compdata(1024), rawdata(1024);
     typedef std::vector<char>::size_type size_t;
 
-    if ( compdata.size() < static_cast<size_t>(object.length()) ) {
+    if ( compdata.size() <= static_cast<size_t>(object.length()) ) { // include trailing \0
         size_t sz = ( static_cast<size_t>(object.length()) + 4096 ) & (~static_cast<size_t>(4096-1));
         std::vector<char>( sz ).swap( compdata ); // resize without copying values
         std::vector<char>( sz ).swap( rawdata );
@@ -567,7 +568,7 @@ vcd_sc_signed_trace::write(FILE* f)
     static std::vector<char> compdata(1024), rawdata(1024);
     typedef std::vector<char>::size_type size_t;
 
-    if ( compdata.size() < static_cast<size_t>(object.length()) ) {
+    if ( compdata.size() <= static_cast<size_t>(object.length()) ) { // include trailing \0
         size_t sz = ( static_cast<size_t>(object.length()) + 4096 ) & (~static_cast<size_t>(4096-1));
         std::vector<char>( sz ).swap( compdata ); // resize without copying values
         std::vector<char>( sz ).swap( rawdata );
@@ -832,7 +833,7 @@ vcd_sc_fxnum_trace::write( FILE* f )
     static std::vector<char> compdata(1024), rawdata(1024);
     typedef std::vector<char>::size_type size_t;
 
-    if ( compdata.size() < static_cast<size_t>(object.wl()) ) {
+    if ( compdata.size() <= static_cast<size_t>(object.wl()) ) { // include trailing \0
         size_t sz = ( static_cast<size_t>(object.wl()) + 4096 ) & (~static_cast<size_t>(4096-1));
         std::vector<char>( sz ).swap( compdata ); // resize without copying values
         std::vector<char>( sz ).swap( rawdata );
@@ -902,7 +903,7 @@ vcd_sc_fxnum_fast_trace::write( FILE* f )
     static std::vector<char> compdata(1024), rawdata(1024);
     typedef std::vector<char>::size_type size_t;
 
-    if ( compdata.size() < static_cast<size_t>(object.wl()) ) {
+    if ( compdata.size() <= static_cast<size_t>(object.wl()) ) { // include trailing \0
         size_t sz = ( static_cast<size_t>(object.wl()) + 4096 ) & (~static_cast<size_t>(4096-1));
         std::vector<char>( sz ).swap( compdata ); // resize without copying values
         std::vector<char>( sz ).swap( rawdata );
@@ -1192,7 +1193,7 @@ public:
 protected:
     const int& object;
     int old_value;
-    unsigned mask;
+    const unsigned rem_bits;
 };
 
 
@@ -1201,10 +1202,9 @@ vcd_signed_int_trace::vcd_signed_int_trace(const signed& object_,
 					   const std::string& vcd_name_,
 					   int width_)
 : vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
-  mask(~0U)
+  rem_bits(32 - width_)
 {
     bit_width = width_;
-    if (bit_width < 32) mask = ~(~0U << bit_width);
 }
 
 
@@ -1221,7 +1221,7 @@ void vcd_signed_int_trace::write(FILE* f)
     int bitindex;
 
     // Check for overflow
-    if ((static_cast<unsigned>(object) & mask) != static_cast<unsigned>(object)) {
+    if (((object << rem_bits) >> rem_bits) != object) {
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = 'x';
         }
@@ -1253,7 +1253,7 @@ public:
 protected:
     const short& object;
     short old_value;
-    unsigned short mask;
+    const unsigned rem_bits;
 };
 
 
@@ -1262,10 +1262,9 @@ vcd_signed_short_trace::vcd_signed_short_trace(
 					const std::string& name_,
 					const std::string& vcd_name_,
 					int width_)
-: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(static_cast<unsigned short>(~0U))
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), rem_bits(32 - width_)
 {
     bit_width = width_;
-    if (bit_width < 16) mask = static_cast<unsigned short>(~(~0U << bit_width));
 }
 
 
@@ -1282,7 +1281,7 @@ void vcd_signed_short_trace::write(FILE* f)
     int bitindex;
 
     // Check for overflow
-    if ((static_cast<unsigned short>(object) & mask) != static_cast<unsigned short>(object)) {
+    if (((object << rem_bits) >> rem_bits) != object) {
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = 'x';
         }
@@ -1314,7 +1313,7 @@ public:
 protected:
     const char& object;
     char old_value;
-    unsigned char mask;
+    const unsigned rem_bits;
 };
 
 
@@ -1322,10 +1321,9 @@ vcd_signed_char_trace::vcd_signed_char_trace(const char& object_,
 					     const std::string& name_,
 					     const std::string& vcd_name_,
 					     int width_)
-: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), mask(static_cast<unsigned char>(~0U))
+: vcd_trace(name_, vcd_name_), object(object_), old_value(object_), rem_bits(32 - width_)
 {
     bit_width = width_;
-    if (bit_width < 8) mask = static_cast<unsigned char>(~(~0U << bit_width));
 }
 
 
@@ -1342,7 +1340,7 @@ void vcd_signed_char_trace::write(FILE* f)
     int bitindex;
 
     // Check for overflow
-    if ((static_cast<unsigned char>(object) & mask) != static_cast<unsigned char>(object)) {
+    if (((object << rem_bits) >> rem_bits) != object) {
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = 'x';
         }
@@ -1374,7 +1372,7 @@ public:
 protected:
     const sc_dt::int64& object;
     sc_dt::int64 old_value;
-    sc_dt::uint64 mask;
+    const unsigned rem_bits;
 };
 
 
@@ -1383,10 +1381,9 @@ vcd_int64_trace::vcd_int64_trace(const sc_dt::int64& object_,
 					     const std::string& vcd_name_,
 					     int width_)
 : vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
-  mask((sc_dt::uint64)-1)
+  rem_bits(64 - width_)
 {
     bit_width = width_;
-    if (bit_width < 64)  mask = ~(mask << bit_width);
 }
 
 
@@ -1403,7 +1400,7 @@ void vcd_int64_trace::write(FILE* f)
     int bitindex;
 
     // Check for overflow
-    if ((static_cast<sc_dt::uint64>(object) & mask) != static_cast<sc_dt::uint64>(object))
+    if (((object << rem_bits) >> rem_bits) != object)
     {
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = 'x';
@@ -1534,7 +1531,7 @@ public:
 protected:
     const long& object;
     long old_value;
-    unsigned long mask;
+    const unsigned rem_bits;
 };
 
 
@@ -1543,11 +1540,9 @@ vcd_signed_long_trace::vcd_signed_long_trace(const long& object_,
 					     const std::string& vcd_name_,
 					     int width_)
 : vcd_trace(name_, vcd_name_), object(object_), old_value(object_),
-  mask(~0UL)
+  rem_bits(std::numeric_limits<unsigned long>::digits - width_)
 {
     bit_width = width_;
-    if ( bit_width < static_cast<int>(sizeof(long)*8) )
-        mask = ~(~0UL << bit_width);
 }
 
 
@@ -1564,7 +1559,7 @@ void vcd_signed_long_trace::write(FILE* f)
     int bitindex;
 
     // Check for overflow
-    if ((static_cast<unsigned long>(object) & mask) != static_cast<unsigned long>(object)) {
+    if (((object << rem_bits) >> rem_bits) != object) {
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             rawdata[bitindex] = 'x';
         }
@@ -2142,7 +2137,7 @@ vcd_trace_file::obtain_name()
 vcd_trace_file::~vcd_trace_file()
 {
     unit_type now_units_high, now_units_low;
-    if (get_time_stamp(now_units_high,now_units_low)) {
+    if (is_initialized() && get_time_stamp(now_units_high,now_units_low)) {
         print_time_stamp(now_units_high, now_units_low);
     }
 
