@@ -27,7 +27,11 @@
   CHANGE LOG AT END OF FILE
  *****************************************************************************/
 
-#include "sysc/utils/sc_iostream.h"
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <fstream>
+
 #include "sysc/kernel/sc_process.h"
 #include "sysc/kernel/sc_simcontext_int.h"
 #include "sysc/utils/sc_stop_here.h"
@@ -91,23 +95,91 @@ const std::string sc_report_compose_message(const sc_report& rep)
 
     return str;
 }
-bool sc_report_close_default_log();
 
-static ::std::ofstream* log_stream = 0;
-static
-struct auto_close_log
+//
+// Private class to handle log files
+//
+class sc_log_file_handle
 {
-    ~auto_close_log()
-    {
-	sc_report_close_default_log();
-    }
-} auto_close;
+protected:
+	// not CopyConstructible
+	sc_log_file_handle(sc_log_file_handle const &);	
+	
+	// not CopyAssignable
+	void operator=(sc_log_file_handle const &);
 
-const char* sc_report::get_process_name() const
+public:
+	sc_log_file_handle();
+	sc_log_file_handle(const char *);
+	void update_file_name(const char *);
+	bool release();
+	::std::ofstream& operator*();
+
+private:
+	std::string log_file_name;
+	::std::ofstream log_stream; 
+};
+
+sc_log_file_handle::sc_log_file_handle()
+{}
+
+sc_log_file_handle::sc_log_file_handle(const char * fname)
+: log_file_name(fname)
+, log_stream(fname)
+{}
+
+void
+sc_log_file_handle::update_file_name(const char * fname)
 {
-	return process ? process->name() : 0;
+	if (!fname)
+	{
+		release();
+	}
+	else //fname != NULL
+	{
+		if (log_file_name.empty())
+		{
+			if (log_stream.is_open()) //should be closed already
+				log_stream.close();
+			log_file_name = fname;
+			log_stream.open(fname);
+		}
+		else // log_file_name not empty
+		{
+			// filename changed?
+			if (log_file_name != fname)
+			{
+				// new filename
+				release();
+				log_file_name = fname;
+				log_stream.open(fname);
+			}
+			else
+			{
+				// nothing to do
+			}
+		}
+	}
 }
 
+bool
+sc_log_file_handle::release()
+{
+	if (log_stream.is_open())
+	{
+		log_stream.close();
+		log_file_name.clear();
+		return false;
+	}
+	return true;
+}
+
+::std::ofstream& 
+sc_log_file_handle::operator*()
+{ return log_stream;	}
+
+static sc_log_file_handle log_stream;
+
 
 //
 // The official handler of the exception reporting
@@ -122,8 +194,7 @@ void sc_report_handler::default_handler(const sc_report& rep,
 
     if ( (actions & SC_LOG) && get_log_file_name() )
     {
-	if ( !log_stream )
-	    log_stream = new ::std::ofstream(get_log_file_name()); // ios::trunc
+		log_stream.update_file_name(get_log_file_name());
 
 	*log_stream << rep.get_time() << ": "
 	    << sc_report_compose_message(rep) << ::std::endl;
@@ -137,7 +208,7 @@ void sc_report_handler::default_handler(const sc_report& rep,
 	sc_interrupt_here(rep.get_msg_type(), rep.get_severity());
 
     if ( actions & SC_ABORT )
-	abort();
+        sc_abort();
 
     if ( actions & SC_THROW ) {
         sc_process_b* proc_p = sc_get_current_process_b();
@@ -150,14 +221,10 @@ void sc_report_handler::default_handler(const sc_report& rep,
 // not documented, but available
 bool sc_report_close_default_log()
 {
-    delete log_stream;
+    bool ret = log_stream.release();
     sc_report_handler::set_log_file_name(NULL);
 
-    if ( !log_stream )
-	return false;
-
-    log_stream = 0;
-    return true;
+    return ret;
 }
 
 int sc_report_handler::get_count(sc_severity severity_) 
@@ -193,6 +260,9 @@ int sc_report_handler::get_count(const char* msg_type_, sc_severity severity_)
 
 sc_msg_def * sc_report_handler::mdlookup(const char * msg_type_)
 {
+    if( !msg_type_ ) // if msg_type is NULL, report unknown error
+        msg_type_ = SC_ID_UNKNOWN_ERROR_;
+
     for ( msg_def_items * item = messages; item; item = item->next )
     {
 	for ( int i = 0; i < item->count; ++i )
@@ -419,7 +489,7 @@ sc_msg_def * sc_report_handler::add_msg_type(const char * msg_type_)
     }
     else
     {
-	delete items->md;
+	delete [] items->md;
 	delete items;
 	return 0;
     }
@@ -548,9 +618,18 @@ sc_actions sc_report_handler::force()
     return force(0);
 }
 
-void sc_report_handler::set_handler(sc_report_handler_proc handler_)
+sc_report_handler_proc
+sc_report_handler::set_handler(sc_report_handler_proc handler_)
 {
+    sc_report_handler_proc old = handler;
     handler = handler_ ? handler_: &sc_report_handler::default_handler;
+    return old;
+}
+
+sc_report_handler_proc
+sc_report_handler::get_handler()
+{
+    return handler;
 }
 
 sc_report* sc_report_handler::get_cached_report()
@@ -690,17 +769,33 @@ sc_report_handler::msg_def_items * sc_report_handler::messages =
     &sc_report_handler::msg_terminator;
 
 
+sc_actions sc_report_handler::catch_actions = SC_DEFAULT_CATCH_ACTIONS;
+
+sc_actions sc_report_handler::set_catch_actions(sc_actions act)
+{
+    //sc_assert( !(act | SC_THROW) ); // allow SC_THROW?
+    sc_actions old = catch_actions;
+    catch_actions = act;
+    return old;
+}
+
+sc_actions sc_report_handler::get_catch_actions()
+{
+    return catch_actions;
+}
+
 //
 // predefined messages
 //
 
-const char SC_ID_REGISTER_ID_FAILED_[] = "register_id failed";
-const char SC_ID_UNKNOWN_ERROR_[]      = "unknown error";
-const char SC_ID_WITHOUT_MESSAGE_[]    = "";
-const char SC_ID_NOT_IMPLEMENTED_[]    = "not implemented";
-const char SC_ID_INTERNAL_ERROR_[]     = "internal error";
-const char SC_ID_ASSERTION_FAILED_[]   = "assertion failed";
-const char SC_ID_OUT_OF_BOUNDS_[]      = "out of bounds";
+SC_API const char SC_ID_REGISTER_ID_FAILED_[] = "register_id failed";
+SC_API const char SC_ID_UNKNOWN_ERROR_[]      = "unknown error";
+SC_API const char SC_ID_WITHOUT_MESSAGE_[]    = "";
+SC_API const char SC_ID_NOT_IMPLEMENTED_[]    = "not implemented";
+SC_API const char SC_ID_INTERNAL_ERROR_[]     = "internal error";
+SC_API const char SC_ID_ASSERTION_FAILED_[]   = "assertion failed";
+SC_API const char SC_ID_OUT_OF_BOUNDS_[]      = "out of bounds";
+SC_API const char SC_ID_ABORT_[]              = "simulation aborted";
 
 #define DEFINE_MSG(id,n)                                                     \
     {                                                                        \
@@ -718,7 +813,9 @@ static sc_msg_def default_msgs[] = {
     DEFINE_MSG(SC_ID_NOT_IMPLEMENTED_, 2),
     DEFINE_MSG(SC_ID_INTERNAL_ERROR_, 3),
     DEFINE_MSG(SC_ID_ASSERTION_FAILED_, 4),
-    DEFINE_MSG(SC_ID_OUT_OF_BOUNDS_, 5)
+    DEFINE_MSG(SC_ID_OUT_OF_BOUNDS_, 5),
+
+    DEFINE_MSG(SC_ID_ABORT_, 99)
 };
 
 sc_report_handler::msg_def_items sc_report_handler::msg_terminator =
@@ -729,6 +826,23 @@ sc_report_handler::msg_def_items sc_report_handler::msg_terminator =
     NULL
 };
 
+
+void
+sc_abort()
+{
+    SC_REPORT_INFO(SC_ID_ABORT_, 0);
+    abort();
+}
+
+
+void
+sc_assertion_failed(const char* msg, const char* file, int line)
+{
+    sc_report_handler::report( SC_FATAL, SC_ID_ASSERTION_FAILED_
+                             , msg, file, line );
+    sc_abort();
+}
+
 } // namespace sc_core
 
 // $Log: sc_report_handler.cpp,v $
