@@ -28,8 +28,10 @@
 
 #ifndef SC_TEMPORARY_H
 #define SC_TEMPORARY_H
+#include "sysc/kernel/sc_cmnhdr.h"
 
 #include <cstddef>                // std::size_t
+#include <cstring>                // for memset
 
 namespace sc_core {
 
@@ -69,7 +71,7 @@ namespace sc_core {
 //   a heap of the specified number of bytes.
 //       heap_size = number of bytes to allocate for the heap.
 //------------------------------------------------------------------------------
-class sc_byte_heap {
+class SC_API sc_byte_heap {
   public:
     char*  m_bgn_p;  // Beginning of heap storage.
     char*  m_end_p;  // End of heap storage.
@@ -93,6 +95,7 @@ class sc_byte_heap {
     {
         delete [] m_bgn_p;
         m_bgn_p = new char[heap_size];
+        std::memset(m_bgn_p, 0, heap_size);
         m_end_p = &m_bgn_p[heap_size];
         m_next_p = m_bgn_p;
     }
